@@ -11,6 +11,8 @@ SLEEP_INTERVAL = 660  # interval required to let vehicle sleep; based on testing
 DRIVING_INTERVAL = 60  # interval when driving detected
 UPDATE_INTERVAL = 300  # Default polling interval for vehicle
 WEBSOCKET_TIMEOUT = 11  # time for websocket to timeout
+WAKE_TIMEOUT = 60  # max time to wait for vehicle to wake
+WAKE_CHECK_INTERVAL = 2  # wait period between wake checks after a wake request
 RELEASE_NOTES_URL = "https://teslascope.com/teslapedia/software/"
 AUTH_DOMAIN = "https://auth.tesla.com"
 API_URL = "https://owner-api.teslamotors.com"
